@@ -1,9 +1,6 @@
 """Visp calibration pipeline parameters."""
 import logging
 from typing import Any
-from typing import Dict
-from typing import List
-from typing import Optional
 
 import numpy as np
 from dkist_processing_common.models.parameters import ParameterBase
@@ -15,8 +12,8 @@ class VispParameters(ParameterBase):
 
     def __init__(
         self,
-        input_dataset_parameters: Dict[str, List[InputDatasetParameterValue]],
-        wavelength: Optional[float] = None,
+        input_dataset_parameters: dict[str, list[InputDatasetParameterValue]],
+        wavelength: float | None = None,
     ):
         super().__init__(input_dataset_parameters)
         self._wavelength = wavelength
@@ -26,6 +23,40 @@ class VispParameters(ParameterBase):
         """Pixel location of the border between ViSP beams."""
         return self._find_most_recent_past_value("visp_beam_border")
 
+    @property
+    def background_num_spatial_bins(self) -> int:
+        """Return number of spatial bins to use when computing background light."""
+        return self._find_most_recent_past_value("visp_background_num_spatial_bins")
+
+    @property
+    def background_wavelength_subsample_factor(self) -> int:
+        """Return the sub-sampling factor used to reduce the number of wavelength samples."""
+        return self._find_most_recent_past_value("visp_background_wavelength_subsample_factor")
+
+    @property
+    def background_num_fit_iterations(self) -> int:
+        """Maximum number of fit iterations used to fit the background light."""
+        return self._find_most_recent_past_value("visp_background_num_fit_iterations")
+
+    @property
+    def background_continuum_index(self) -> list:
+        """Return indices of a region to use when normalizing modulated polcals in the background task."""
+        return self._find_parameter_closest_wavelength("visp_background_continuum_index")
+
+    @property
+    def lamp_high_pass_filter_width_px(self) -> float:
+        """Sigma (in pixels) of the gaussian filter used for pseudo unsharp masking of the lamp gain images.
+
+        Features that are smaller than the ~sigma will be "passed" by the filter, so a larger sigma corresponds to a
+        larger "width" in the HP filter.
+        """
+        return self._find_parameter_closest_wavelength("visp_lamp_high_pass_filter_width_px")
+
+    @property
+    def gain_hairline_fraction(self):
+        """Relative threshold below median used to identify slit positions covered by the hairlines."""
+        return self._find_most_recent_past_value("visp_gain_hairline_fraction")
+
     @property
     def geo_num_otsu(self):
         """Max number of times to run Otsu's method when making a binary image from solar spectra."""
@@ -46,6 +77,11 @@ class VispParameters(ParameterBase):
         """Maximum angle (in radians) to consider for spectral tilt calculation."""
         return self._find_most_recent_past_value("visp_geo_theta_max")
 
+    @property
+    def geo_max_beam_2_angle_refinement(self) -> float:
+        """Maximum allowable refinement to the beam 2 spectral tilt angle, in radians."""
+        return self._find_most_recent_past_value("visp_geo_max_beam_2_angle_refinement")
+
     @property
     def geo_upsample_factor(self):
         """Pixel precision (1/upsample_factor) to use during phase matching of beam/modulator images."""
@@ -64,12 +100,7 @@ class VispParameters(ParameterBase):
     @property
     def solar_spectral_avg_window(self):
         """Pixel sigma of Gaussian kernal used to compute characteristic solar spectra."""
-        return self._find_most_recent_past_value("visp_solar_spectral_avg_window")
-
-    @property
-    def solar_hairline_fraction(self):
-        """Relative threshold below median used to identify slit positions covered by the hairlines."""
-        return self._find_most_recent_past_value("visp_solar_hairline_fraction")
+        return self._find_parameter_closest_wavelength("visp_solar_spectral_avg_window")
 
     @property
     def solar_zone_prominence(self):
@@ -119,6 +150,14 @@ class VispParameters(ParameterBase):
         """
         return self._find_most_recent_past_value("visp_polcal_demod_upsample_order")
 
+    @property
+    def pac_remove_linear_I_trend(self) -> bool:
+        """Flag that determines if a linear intensity trend is removed from the whole PolCal CS.
+
+        The trend is fit using the average flux in the starting and ending clear steps.
+        """
+        return self._find_most_recent_past_value("visp_pac_remove_linear_I_trend")
+
     @property
     def pac_fit_mode(self):
         """Name of set of fitting flags to use during PAC Calibration Unit parameter fits."""
