@@ -1,11 +1,12 @@
 """Visp geometric calibration task."""
 import logging
 from typing import Generator
-from typing import Optional
 
 import numpy as np
 import peakutils as pku
 import scipy.ndimage as spnd
+import scipy.optimize as spo
+import skimage.metrics as skim
 import skimage.registration as skir
 from dkist_processing_common.tasks.mixin.quality import QualityMixin
 from dkist_processing_math.arithmetic import subtract_array_from_arrays
@@ -14,7 +15,6 @@ from dkist_processing_math.statistics import average_numpy_arrays
 from dkist_processing_math.transform import do_hough
 from dkist_processing_math.transform import make_binary
 from scipy.fft import fftn
-from scipy.fft import ifftn
 from scipy.optimize import minimize
 from skimage.registration._phase_cross_correlation import _upsampled_dft
 
@@ -38,7 +38,7 @@ class GeometricCalibration(
     """
     Task class for computing the spectral geometry. Geometry is represented by three quantities.
 
-      - Angle - The angle between slit hairlines and pixel axes. A one dimensional array with two elements- one for each beam.
+      - Angle - The angle (in radians) between slit hairlines and pixel axes. A one dimensional array with two elements- one for each beam.
 
       - State_offset - The [x, y] shift between each modstate in each beam and a fiducial modstate. For each modstate and each beam two state offset values are computed.
 
@@ -84,6 +84,13 @@ class GeometricCalibration(
                     logging.info(f"Compute and write angle for {beam = }")
                     # TODO: Why not use Lamp gains for angle measurement? They don't have strong spectral lines.
                     angle = self.compute_beam_angle(beam=beam)
+                    if beam == 2:
+                        with self.apm_processing_step("Refining beam 2 angle"):
+                            logging.info("Refining beam 2 angle")
+                            ang_corr = self.refine_beam2_angle(angle)
+                        logging.info(f"Beam 2 angle refinement = {np.rad2deg(ang_corr)} deg")
+                        angle += ang_corr
+                        logging.info(f"Final beam 2 angle = {np.rad2deg(angle)} deg")
                     self.write_angle(angle=angle, beam=beam)
 
                 for modstate in range(1, self.constants.num_modstates + 1):
@@ -289,7 +296,7 @@ class GeometricCalibration(
         Returns
         -------
         float
-            Beam angle
+            Beam angle in radians
         """
         logging.info(f"Finding angle using modstate 1 from beam {beam}")
         beam_array = self.find_brightest_modstate(beam=beam)
@@ -306,6 +313,77 @@ class GeometricCalibration(
         logging.info(f"Beam angle for {beam=}: {np.rad2deg(peak_theta):0.3f} deg")
         return peak_theta
 
+    def refine_beam2_angle(self, beam_2_init_angle: float) -> float:
+        """Find the angular adjustment needed to align beam 2's spectra with beam 1.
+
+        Because this angle will apply to all modulation states the comparison is made between averages over all
+        modulation states of beam 1 and beam 2. First, a rough, global, offset is computed and then a minimizer
+        minimizes the difference between the beams after applying first a rotation, then an offset to beam 2.
+
+        To reduce the influence of low-frequency differences between the beams (e.g., optical gain resonse) the beam
+        averages are first put through a high-pass filter that preserves hairlines and spectral features (which are
+        sharp).
+        """
+        beam_1_angle = self.intermediate_frame_helpers_load_angle(beam=1)
+        beam_1_hpf, beam_1_mask = self.prep_refine_ang_input(beam=1, angle=beam_1_angle)
+        beam_2_hpf, beam_2_mask = self.prep_refine_ang_input(beam=2, angle=beam_2_init_angle)
+
+        logging.info("Computing rough-guess average beam offset")
+        x_init, y_init = self.compute_single_state_offset(
+            beam_1_hpf, beam_2_hpf, fiducial_mask=beam_1_mask, array_mask=beam_2_mask
+        )
+
+        logging.info("Fitting beam 2 angle refinement")
+        res = spo.minimize(
+            self.refine_ang_func,
+            np.array(
+                [
+                    x_init,
+                    y_init,
+                    0.0,
+                ]
+            ),
+            args=(beam_1_hpf, beam_2_hpf),
+            method="nelder-mead",
+        )
+
+        angle = res.x[2]
+        if abs(angle) > self.parameters.geo_max_beam_2_angle_refinement:
+            logging.info(f"Refining angle is too large ({np.rad2deg(angle)} deg). Not using it.")
+            return 0.0
+
+        return angle
+
+    def prep_refine_ang_input(self, beam: int, angle: float) -> tuple[np.ndarray, np.ndarray]:
+        """Prepare an averaged, high-pass-filtered array for a single beam.
+
+        Average is over all modulation states.
+        """
+        logging.info(f"Prepping beam {beam} data for angle refinement")
+        arrays = list(
+            self.intermediate_frame_helpers_load_intermediate_arrays(beam=beam, task="GC_BASIC")
+        )
+        angle_corr_arrays = self.corrections_correct_geometry(arrays, angle=angle)
+        modstate_avg = np.mean(np.stack(angle_corr_arrays), axis=0)
+
+        mask = self.compute_offset_mask(modstate_avg)
+        high_pass_filtered_array = self.high_pass_filter_array(modstate_avg)
+
+        return high_pass_filtered_array, mask
+
+    @staticmethod
+    def refine_ang_func(x, target, image) -> float:
+        """Rotate and shift beam 2 then compare for similarity with beam 1."""
+        xshift, yshift, ang = x
+        rotated = next(GeometricCalibration.corrections_correct_geometry(image, angle=ang))
+        shifted = next(
+            GeometricCalibration.corrections_correct_geometry(
+                rotated, shift=np.array([xshift, yshift])
+            )
+        )
+        metric = 1 - skim.structural_similarity(target, shifted)
+        return metric
+
     def remove_beam_angle(self, angle: float, beam: int, modstate: int) -> np.ndarray:
         """
         Rotate a single modstate and beam's data by the beam angle.
@@ -313,7 +391,7 @@ class GeometricCalibration(
         Parameters
         ----------
         angle : float
-            The beam angle for the current modstate
+            The beam angle (in radians) for the current modstate
         beam : int
             The current beam being processed
         modstate : int
@@ -324,8 +402,7 @@ class GeometricCalibration(
         np.ndarray
             Array corrected for beam angle
         """
-        # TODO: Figure out why the type of basic_corrected_data was >f8 prior to conversion
-        beam_mod_array = self.basic_corrected_data(beam=beam, modstate=modstate).astype(np.float64)
+        beam_mod_array = self.basic_corrected_data(beam=beam, modstate=modstate)
         corrected_array = next(self.corrections_correct_geometry(beam_mod_array, angle=angle))
         return corrected_array
 
@@ -351,14 +428,15 @@ class GeometricCalibration(
         """
         if beam == 1 and modstate == 1:
             logging.info("Set fiducial array")
-            self.fiducial_array = array
+            self.fiducial_array = self.high_pass_filter_array(array)
             self.fiducial_mask = self.compute_offset_mask(array)
             return np.zeros(2)
 
+        hpf_array = self.high_pass_filter_array(array)
         array_mask = self.compute_offset_mask(array)
         shift = self.compute_single_state_offset(
             fiducial_array=self.fiducial_array,
-            array=array,
+            array=hpf_array,
             upsample_factor=self.parameters.geo_upsample_factor,
             fiducial_mask=self.fiducial_mask,
             array_mask=array_mask,
@@ -478,26 +556,21 @@ class GeometricCalibration(
         ## Subtract the average so we shift my a minimal amount
         beam_shifts -= np.nanmean(beam_shifts)
 
-        ## Finally, interpolate out-of-range shifts
+        ## Finally, fit the shifts and return the resulting polynomial
         nan_idx = np.isnan(beam_shifts)
-        logging.info(
-            f"Interpolating {np.sum(nan_idx)} ({np.sum(nan_idx) / num_spec * 100:0.2f} %) out-of-range pixels"
-        )
         poly = np.poly1d(
             np.polyfit(np.arange(num_spec)[~nan_idx], beam_shifts[~nan_idx], poly_fit_order)
         )
-        nan_x = np.arange(num_spec)[nan_idx]
-        beam_shifts[nan_idx] = poly(nan_x)
 
-        return beam_shifts
+        return poly(np.arange(num_spec))
 
     @staticmethod
     def compute_single_state_offset(
         fiducial_array: np.ndarray,
         array: np.ndarray,
         upsample_factor: float = 1000.0,
-        fiducial_mask: Optional[np.ndarray] = None,
-        array_mask: Optional[np.ndarray] = None,
+        fiducial_mask: np.ndarray | None = None,
+        array_mask: np.ndarray | None = None,
     ) -> np.ndarray:
         """
         Find the (x, y) shift between the current beam and the reference beam.
@@ -616,12 +689,17 @@ class GeometricCalibration(
         logging.info(f"Brightest modstate is {brightest_modstate}")
         return brightest_array
 
+    @staticmethod
+    def high_pass_filter_array(array: np.ndarray) -> np.ndarray:
+        """Perform a simple high-pass filter to accentuate narrow features (hairlines and spectra)."""
+        return array / spnd.gaussian_filter(array, sigma=5)
+
     @staticmethod
     def shift_chisq(par: np.ndarray, ref_spec: np.ndarray, spec: np.ndarray) -> float:
         """
         Goodness of fit calculation for a simple shift. Uses simple chisq as goodness of fit.
 
-        Less robust than SolarCalibration's `reshift`, but much faster.
+        Less robust than SolarCalibration's `refine_shift`, but much faster.
 
         Parameters
         ----------
@@ -652,7 +730,7 @@ class GeometricCalibration(
         Parameters
         ----------
         angle : float
-            The beam angle for the current modstate
+            The beam angle (radians) for the current modstate
 
         beam : int
             The current beam being processed
