@@ -2,10 +2,8 @@
 import logging
 from collections import defaultdict
 from dataclasses import dataclass
+from functools import cached_property
 from typing import Iterable
-from typing import Optional
-from typing import Tuple
-from typing import Union
 
 import numpy as np
 from astropy.io import fits
@@ -15,7 +13,6 @@ from dkist_processing_common.tasks.mixin.quality import QualityMixin
 from dkist_processing_math.arithmetic import divide_arrays_by_array
 from dkist_processing_math.arithmetic import subtract_array_from_arrays
 from dkist_processing_math.statistics import average_numpy_arrays
-from dkist_processing_pac.optics.telescope import get_TM_db_location
 from dkist_processing_pac.optics.telescope import Telescope
 
 from dkist_processing_visp.models.tags import VispTag
@@ -33,11 +30,51 @@ class CalibrationCollection:
     """Dataclass to hold all calibration objects and allow for easy, property-based access."""
 
     dark: dict
+    background: dict
+    lamp_gain: dict
     solar_gain: dict
     angle: dict
     state_offset: dict
     spec_shift: dict
-    demod_matrices: Optional[dict]
+    demod_matrices: dict | None
+
+    @cached_property
+    def beams_overlap_slice(self) -> tuple[slice, slice]:
+        """
+        Compute array slices that will extract the largest region with overlap from both beams.
+
+        This is done by considering that state offset values computed by the GeometricCalibration task. Any sub-pixel
+        overlaps are rounded to the next integer that still guarantees overlap.
+        """
+        logging.info("Computing beam overlap slices")
+        # This will be a flat list of (x, y) pairs for all modstates and beams
+        flat_offsets = sum([list(i.values()) for i in self.state_offset.values()], [])
+        # Split out into an x list and a y list
+        all_x_shifts, all_y_shifts = zip(*flat_offsets)
+        all_x_shifts = np.array(all_x_shifts)
+        all_y_shifts = np.array(all_y_shifts)
+
+        logging.info(f"All x shifts: {all_x_shifts}")
+        logging.info(f"All y shifts: {all_y_shifts}")
+
+        # The np.abs is necessary to find that largest magnitude shift
+        max_x = all_x_shifts[np.argmax(np.abs(all_x_shifts))]
+        max_y = all_y_shifts[np.argmax(np.abs(all_y_shifts))]
+
+        # The overlap region is simply the region defined by the largest slices in from the edges
+        if max_x < 0:
+            # Negative shifts mean we need to chop in from the "end" of the array
+            x_slice = slice(None, int(np.floor(max_x)))
+        else:
+            # Positive shifts mean we start in from the "start" of the array
+            x_slice = slice(int(np.ceil(max_x)), None)
+
+        if max_y < 0:
+            y_slice = slice(None, int(np.floor(max_y)))
+        else:
+            y_slice = slice(int(np.ceil(max_y)), None)
+
+        return x_slice, y_slice
 
 
 class ScienceCalibration(
@@ -106,6 +143,8 @@ class ScienceCalibration(
         Doing this once here prevents lots of reads as we reduce the science data.
         """
         dark_dict = defaultdict(dict)
+        background_dict = dict()
+        lamp_dict = defaultdict(dict)
         solar_dict = defaultdict(dict)
         angle_dict = dict()
         state_offset_dict = defaultdict(dict)
@@ -125,6 +164,17 @@ class ScienceCalibration(
 
                 dark_dict[VispTag.beam(beam)][VispTag.exposure_time(exp_time)] = dark_array
 
+            # Residual background light
+            ###########################
+            try:
+                background_dict[
+                    VispTag.beam(beam)
+                ] = self.intermediate_frame_helpers_load_background_array(beam=beam)
+            except StopIteration:
+                if self.constants.correct_for_polarization:
+                    raise ValueError(f"No matching background light found for {beam = }")
+                background_dict[VispTag.beam(beam)] = np.zeros(dark_array.shape)
+
             # Angle
             #######
             angle_dict[VispTag.beam(beam)] = self.intermediate_frame_helpers_load_angle(beam=beam)
@@ -143,6 +193,14 @@ class ScienceCalibration(
                 ] = self.intermediate_frame_helpers_load_demod_matrices(beam_num=beam)
 
             for modstate in range(1, self.constants.num_modstates + 1):
+                # Lamp
+                ######
+                lamp_dict[VispTag.beam(beam)][
+                    VispTag.modstate(modstate)
+                ] = self.intermediate_frame_helpers_load_lamp_gain_array(
+                    beam=beam, modstate=modstate
+                )
+
                 # Solar
                 #######
                 solar_dict[VispTag.beam(beam)][
@@ -159,6 +217,8 @@ class ScienceCalibration(
 
         return CalibrationCollection(
             dark=dark_dict,
+            background=background_dict,
+            lamp_gain=lamp_dict,
             solar_gain=solar_dict,
             angle=angle_dict,
             state_offset=state_offset_dict,
@@ -170,7 +230,7 @@ class ScienceCalibration(
         """
         Completely calibrate all science frames.
 
-        - Apply all dark, gain, geometric corrections
+        - Apply all dark, background, gain, geometric corrections
         - Demodulate if needed
         - Combine beams
         - Apply telescope correction, if needed
@@ -217,7 +277,7 @@ class ScienceCalibration(
 
                     with self.apm_processing_step("Combining beams"):
                         logging.info("Combining beams")
-                        calibrated = self.combine_beams(beam_storage, header_storage)
+                        calibrated = self.combine_beams(beam_storage, header_storage, calibrations)
 
                     if self.constants.correct_for_polarization:
                         with self.apm_processing_step("Correcting telescope polarization"):
@@ -227,7 +287,9 @@ class ScienceCalibration(
                     # Save the final output files
                     with self.apm_writing_step("Writing calibrated arrays"):
                         logging.info("Writing calibrated arrays")
-                        self.write_calibrated_array(calibrated, map_scan=map_scan)
+                        self.write_calibrated_array(
+                            calibrated, map_scan=map_scan, calibrations=calibrations
+                        )
 
     def process_polarimetric_modstates(
         self,
@@ -236,7 +298,7 @@ class ScienceCalibration(
         map_scan: int,
         exp_time: float,
         calibrations: CalibrationCollection,
-    ) -> Tuple[np.ndarray, fits.Header]:
+    ) -> tuple[np.ndarray, fits.Header]:
         """
         Process a single polarimetric beam as much as is possible.
 
@@ -271,9 +333,11 @@ class ScienceCalibration(
 
         return intermediate_array, intermediate_header
 
-    def combine_beams(self, array_dict: dict, header_dict: dict) -> VispL0FitsAccess:
+    def combine_beams(
+        self, array_dict: dict, header_dict: dict, calibrations: CalibrationCollection
+    ) -> VispL0FitsAccess:
         """
-        Average all beams together.
+        Average all beams together and chop the resulting frame to just the region of overlap.
 
         Also complain if the inputs are strange.
         """
@@ -290,12 +354,21 @@ class ScienceCalibration(
 
         avg_array = average_numpy_arrays(array_list)
 
-        hdu = fits.ImageHDU(data=avg_array, header=headers[0])
+        x_slice, y_slice = calibrations.beams_overlap_slice
+        logging.info(f"Trimming non-overlapping beam edges by ({x_slice}, {y_slice})")
+        cut_array = avg_array[x_slice, y_slice]
+
+        hdu = fits.ImageHDU(data=cut_array, header=headers[0])
         obj = VispL0FitsAccess(hdu=hdu, auto_squeeze=False)
 
         return obj
 
-    def write_calibrated_array(self, calibrated_object: VispL0FitsAccess, map_scan: int) -> None:
+    def write_calibrated_array(
+        self,
+        calibrated_object: VispL0FitsAccess,
+        map_scan: int,
+        calibrations: CalibrationCollection,
+    ) -> None:
         """
         Write out calibrated science frames.
 
@@ -310,12 +383,15 @@ class ScienceCalibration(
         map_scan
             The current map scan. Needed because it's not a header key
         """
+        # We only need to compute the header once
+        #  (In fact we *need* to only compute it once because we update WCS values in place and running this function
+        #   more than once would result in incorrect WCS info).
+        final_header = self._update_calibrated_header(
+            calibrated_object.header, map_scan=map_scan, calibrations=calibrations
+        )
         if self.constants.correct_for_polarization:  # Write all 4 stokes params
             for i, stokes_param in enumerate(self.constants.stokes_params):
                 final_data = self._re_dummy_data(calibrated_object.data[:, :, i])
-                final_header = self._update_calibrated_header(
-                    calibrated_object.header, map_scan=map_scan
-                )
                 self.write_cal_array(
                     data=final_data,
                     header=final_header,
@@ -325,9 +401,6 @@ class ScienceCalibration(
                 )
         else:  # Only write stokes I
             final_data = self._re_dummy_data(calibrated_object.data)
-            final_header = self._update_calibrated_header(
-                calibrated_object.header, map_scan=map_scan
-            )
             self.write_cal_array(
                 data=final_data,
                 header=final_header,
@@ -344,15 +417,16 @@ class ScienceCalibration(
         map_scan: int,
         exp_time: float,
         calibrations: CalibrationCollection,
-    ) -> Tuple[np.ndarray, fits.Header]:
+    ) -> tuple[np.ndarray, fits.Header]:
         """
         Apply basic corrections to a single frame.
 
         Generally the algorithm is:
             1. Dark correct the array
-            2. Solar Gain correct the array
-            3. Geo correct the array
-            4. Spectral correct array
+            2. Background correct the array
+            3. Solar Gain correct the array
+            4. Geo correct the array
+            5. Spectral correct array
 
         Parameters
         ----------
@@ -375,6 +449,8 @@ class ScienceCalibration(
         """
         # Extract calibrations
         dark_array = calibrations.dark[VispTag.beam(beam)][VispTag.exposure_time(exp_time)]
+        background_array = calibrations.background[VispTag.beam(beam)]
+        lamp_gain_array = calibrations.lamp_gain[VispTag.beam(beam)][VispTag.modstate(modstate)]
         solar_gain_array = calibrations.solar_gain[VispTag.beam(beam)][VispTag.modstate(modstate)]
         angle = calibrations.angle[VispTag.beam(beam)]
         spec_shift = calibrations.spec_shift[VispTag.beam(beam)]
@@ -402,12 +478,22 @@ class ScienceCalibration(
         # Dark correction
         dark_corrected_array = next(subtract_array_from_arrays(observe_data, dark_array))
 
-        # Gain correction
-        gain_corrected_array = next(divide_arrays_by_array(dark_corrected_array, solar_gain_array))
+        # Residual background correction
+        background_corrected_array = next(
+            subtract_array_from_arrays(dark_corrected_array, background_array)
+        )
+
+        # Lamp gain correction
+        lamp_corrected_array = next(
+            divide_arrays_by_array(background_corrected_array, lamp_gain_array)
+        )
+
+        # Solar gain correction
+        solar_corrected_array = next(divide_arrays_by_array(lamp_corrected_array, solar_gain_array))
 
         # Geo correction
         geo_corrected_array = next(
-            self.corrections_correct_geometry(gain_corrected_array, state_offset, angle)
+            self.corrections_correct_geometry(solar_corrected_array, state_offset, angle)
         )
 
         # Geo correction pt 2: spectral curvature
@@ -463,7 +549,7 @@ class ScienceCalibration(
         return inst_demod_obj
 
     @staticmethod
-    def _compute_date_keys(headers: Union[Iterable[fits.Header], fits.Header]) -> fits.Header:
+    def _compute_date_keys(headers: Iterable[fits.Header] | fits.Header) -> fits.Header:
         """
         Generate correct DATE-??? header keys from a set of input headers.
 
@@ -508,7 +594,9 @@ class ScienceCalibration(
         logging.debug(f"Adding dummy WCS dimension to array with shape {data.shape}")
         return data[None, :, :]
 
-    def _update_calibrated_header(self, header: fits.Header, map_scan: int) -> fits.Header:
+    def _update_calibrated_header(
+        self, header: fits.Header, map_scan: int, calibrations: CalibrationCollection
+    ) -> fits.Header:
         """
         Update calibrated headers with any information gleaned during science calibration.
 
@@ -526,9 +614,33 @@ class ScienceCalibration(
         -------
         fits.Header
         """
+        # Update the map scan number
         header["VSPNMAPS"] = self.constants.num_map_scans
         header["VSPMAP"] = map_scan
 
+        # Adjust the CRPIX values if the beam overlap slicing chopped from the start of the array
+        x_slice, y_slice = calibrations.beams_overlap_slice
+        if x_slice.start:
+            # .start will only be non-None if the slice is from the start. In this case we need to update the WCS
+            logging.info(
+                f"Adjusting CRPIX2 from {header['CRPIX2']} to {header['CRPIX2'] - x_slice.start}"
+            )
+            header["CRPIX2"] = header["CRPIX2"] - x_slice.start
+            logging.info(
+                f"Adjusting CRPIX2A from {header['CRPIX2A']} to {header['CRPIX2A'] - x_slice.start}"
+            )
+            header["CRPIX2A"] = header["CRPIX2A"] - x_slice.start
+
+        if y_slice.start:
+            logging.info(
+                f"Adjusting CRPIX1 from {header['CRPIX1']} to {header['CRPIX1'] - y_slice.start}"
+            )
+            header["CRPIX1"] = header["CRPIX1"] - y_slice.start
+            logging.info(
+                f"Adjusting CRPIX1A from {header['CRPIX1A']} to {header['CRPIX1A'] - y_slice.start}"
+            )
+            header["CRPIX1A"] = header["CRPIX1A"] - y_slice.start
+
         return header
 
     def write_cal_array(
