@@ -6,10 +6,6 @@ from dataclasses import is_dataclass
 from random import choice
 from random import randint
 from typing import Iterable
-from typing import Optional
-from typing import Tuple
-from typing import Union
-from unittest.mock import PropertyMock
 
 import numpy as np
 import pytest
@@ -32,8 +28,8 @@ from dkist_processing_visp.models.tags import VispTag
 class VispHeaders(Spec122Dataset):
     def __init__(
         self,
-        dataset_shape: Tuple[int, ...],
-        array_shape: Tuple[int, ...],
+        dataset_shape: tuple[int, ...],
+        array_shape: tuple[int, ...],
         time_delta: 10,
         instrument: str = "visp",
         polarimeter_mode: str = "observe_polarimetric",
@@ -73,8 +69,8 @@ class VispHeaders(Spec122Dataset):
 class VispHeadersValidDarkFrames(VispHeaders):
     def __init__(
         self,
-        dataset_shape: Tuple[int, ...],
-        array_shape: Tuple[int, ...],
+        dataset_shape: tuple[int, ...],
+        array_shape: tuple[int, ...],
         time_delta: float,
         **kwargs,
     ):
@@ -94,8 +90,8 @@ class VispHeadersValidDarkFrames(VispHeaders):
 class VispHeadersValidLampGainFrames(VispHeaders):
     def __init__(
         self,
-        dataset_shape: Tuple[int, ...],
-        array_shape: Tuple[int, ...],
+        dataset_shape: tuple[int, ...],
+        array_shape: tuple[int, ...],
         time_delta: float,
         num_modstates: int,
         modstate: int,
@@ -118,8 +114,8 @@ class VispHeadersValidLampGainFrames(VispHeaders):
 class VispHeadersValidSolarGainFrames(VispHeaders):
     def __init__(
         self,
-        dataset_shape: Tuple[int, ...],
-        array_shape: Tuple[int, ...],
+        dataset_shape: tuple[int, ...],
+        array_shape: tuple[int, ...],
         time_delta: float,
         num_modstates: int,
         modstate: int,
@@ -142,8 +138,8 @@ class VispHeadersValidSolarGainFrames(VispHeaders):
 class VispHeadersValidPolcalFrames(VispHeaders):
     def __init__(
         self,
-        dataset_shape: Tuple[int, ...],
-        array_shape: Tuple[int, ...],
+        dataset_shape: tuple[int, ...],
+        array_shape: tuple[int, ...],
         time_delta: float,
         num_modstates: int,
         modstate: int,
@@ -170,8 +166,8 @@ class VispHeadersValidPolcalFrames(VispHeaders):
 class VispHeadersValidObserveFrames(VispHeaders):
     def __init__(
         self,
-        dataset_shape: Tuple[int, ...],
-        array_shape: Tuple[int, ...],
+        dataset_shape: tuple[int, ...],
+        array_shape: tuple[int, ...],
         time_delta: float,
         num_raster_steps: int,
         raster_step: int,
@@ -200,7 +196,7 @@ def generate_fits_frame(header_generator: Iterable, shape=None) -> fits.HDUList:
 
 
 def generate_full_visp_fits_frame(
-    header_generator: Iterable, data: Optional[np.ndarray] = None
+    header_generator: Iterable, data: np.ndarray | None = None
 ) -> fits.HDUList:
     if data is None:
         data = np.ones(shape=(1, 2000, 2560))
@@ -213,7 +209,7 @@ def generate_full_visp_fits_frame(
 
 
 def generate_214_l0_fits_frame(
-    s122_header: fits.Header, data: Optional[np.ndarray] = None
+    s122_header: fits.Header, data: np.ndarray | None = None
 ) -> fits.HDUList:
     """Convert S122 header into 214 L0"""
     if data is None:
@@ -225,7 +221,7 @@ def generate_214_l0_fits_frame(
 
 
 def generate_214_l1_fits_frame(
-    s122_header: fits.Header, data: Optional[np.ndarray] = None
+    s122_header: fits.Header, data: np.ndarray | None = None
 ) -> fits.HDUList:
     """Convert S122 header into 214 L1 only.
 
@@ -247,7 +243,7 @@ def generate_214_l1_fits_frame(
 class Visp122ObserveFrames(VispHeaders):
     def __init__(
         self,
-        array_shape: Tuple[int, ...],
+        array_shape: tuple[int, ...],
         num_steps: int = 4,
         num_exp_per_step: int = 1,
         num_map_scans: int = 5,
@@ -288,45 +284,20 @@ class VispConstantsDb:
     MAXIMUM_CADENCE: float = 10.0
     VARIANCE_CADENCE: float = 0.0
     WAVELENGTH: float = 588.0
-    LAMP_EXPOSURE_TIMES: Tuple[float] = (100.0,)
-    SOLAR_EXPOSURE_TIMES: Tuple[float] = (1.0,)
-    OBSERVE_EXPOSURE_TIMES: Tuple[float] = (0.01,)
-    POLCAL_EXPOSURE_TIMES: Union[Tuple[float], Tuple] = ()
+    LAMP_EXPOSURE_TIMES: tuple[float] = (100.0,)
+    SOLAR_EXPOSURE_TIMES: tuple[float] = (1.0,)
+    OBSERVE_EXPOSURE_TIMES: tuple[float] = (0.01,)
+    POLCAL_EXPOSURE_TIMES: tuple[float] = ()
     SPECTRAL_LINE: str = "VISP Ca II H"
-    STOKES_PARAMS: Tuple[str] = (
+    STOKES_PARAMS: tuple[str] = (
         "I",
         "Q",
         "U",
         "V",
     )  # A tuple because lists aren't allowed on dataclasses
-
-
-class FakeGQLClient:
-    def __init__(self, *args, **kwargs):
-        pass
-
-    @staticmethod
-    def execute_gql_query(**kwargs):
-        query_base = kwargs["query_base"]
-
-        if query_base == "recipeRuns":
-            return [
-                RecipeRunResponse(
-                    recipeInstanceId=1,
-                    recipeInstance=RecipeInstanceResponse(
-                        recipeId=1,
-                        inputDataset=InputDatasetResponse(
-                            inputDatasetId=1,
-                            isActive=True,
-                            inputDatasetDocument='{"bucket": "bucket-name", "parameters": [{"parameterName": "", "parameterValues": [{"parameterValueId": 1, "parameterValue": "[[1,2,3],[4,5,6],[7,8,9]]", "parameterValueStartDate": "1/1/2000"}]}], "frames": ["objectKey1", "objectKey2", "objectKeyN"]}',
-                        ),
-                    ),
-                )
-            ]
-
-    @staticmethod
-    def execute_gql_mutation(**kwargs):
-        pass
+    AXIS_1_TYPE: str = "HPLT-TAN"
+    AXIS_2_TYPE: str = "AWAV"
+    AXIS_3_TYPE: str = "HPLN-TAN"
 
 
 @pytest.fixture()
@@ -371,15 +342,27 @@ class WavelengthParameter:
 @dataclass
 class VispTestingParameters:
     visp_beam_border: int = 1000
+    visp_background_num_spatial_bins: int = 1
+    visp_background_wavelength_subsample_factor: int = 10
+    visp_background_num_fit_iterations: int = 20
+    visp_background_continuum_index: WavelengthParameter = WavelengthParameter(
+        values=(list(range(190)), list(range(190)), list(range(190)), list(range(190)))
+    )
+    visp_lamp_high_pass_filter_width_px: WavelengthParameter = WavelengthParameter(
+        values=(5, 5, 300, 1)
+    )
+    visp_gain_hairline_fraction: float = 0.8
     visp_geo_num_otsu: int = 5
     visp_geo_num_theta: int = 1500
     visp_geo_theta_min: float = -np.pi / 90
     visp_geo_theta_max: float = np.pi / 90
+    visp_geo_max_beam_2_angle_refinement: float = np.deg2rad(0.1)
     visp_geo_upsample_factor: float = 10.0
     visp_geo_max_shift: float = 40.0
     visp_geo_poly_fit_order: int = 3
-    visp_solar_spectral_avg_window: int = 50
-    visp_solar_hairline_fraction: float = 0.8
+    visp_solar_spectral_avg_window: WavelengthParameter = WavelengthParameter(
+        values=(50, 50, 60, 110)
+    )
     visp_solar_zone_prominence: WavelengthParameter = WavelengthParameter(
         values=(0.2, 0.2, 0.3, 0.2)
     )
@@ -393,6 +376,7 @@ class VispTestingParameters:
     visp_polcal_num_spatial_bins: int = 3
     visp_polcal_num_spectral_bins: int = 1
     visp_polcal_demod_upsample_order: int = 3
+    visp_pac_remove_linear_I_trend: bool = True
     visp_pac_fit_mode: str = "use_M12"
     visp_pac_init_set: str = "OCCal_VIS"
 
@@ -403,9 +387,9 @@ def testing_wavelength() -> float:
 
 
 @pytest.fixture(scope="session")
-def input_dataset_document_with_simple_parameters():
-    def get_input_dataset_document(parameters: VispTestingParameters):
-        dataset_doc_dict = dict(parameters=[])
+def input_dataset_document_simple_parameters_part():
+    def get_input_dataset_parameters_part(parameters: VispTestingParameters):
+        parameters_list = []
         value_id = randint(1000, 2000)
         for pn, pv in asdict(parameters).items():
             if type(pv) is WavelengthParameter:
@@ -418,21 +402,21 @@ def input_dataset_document_with_simple_parameters():
                 }
             ]
             parameter = {"parameterName": pn, "parameterValues": values}
-            dataset_doc_dict["parameters"] += [parameter]
-        return dataset_doc_dict
+            parameters_list.append(parameter)
+        return parameters_list
 
-    return get_input_dataset_document
+    return get_input_dataset_parameters_part
 
 
 @pytest.fixture(scope="session")
 def assign_input_dataset_doc_to_task(
-    input_dataset_document_with_simple_parameters, testing_wavelength
+    input_dataset_document_simple_parameters_part, testing_wavelength
 ):
     def update_task(task, parameters):
-        doc_path = task.scratch.workflow_base_path / "dataset_doc.json"
+        doc_path = task.scratch.workflow_base_path / "dataset_parameters.json"
         with open(doc_path, "w") as f:
-            f.write(json.dumps(input_dataset_document_with_simple_parameters(parameters)))
-        task.tag(doc_path, VispTag.input_dataset())
+            f.write(json.dumps(input_dataset_document_simple_parameters_part(parameters)))
+        task.tag(doc_path, VispTag.input_dataset_parameters())
         task.parameters = VispParameters(
             task.input_dataset_parameters, wavelength=testing_wavelength
         )
