@@ -2,8 +2,6 @@ import json
 import random
 from dataclasses import dataclass
 from datetime import datetime
-from typing import List
-from typing import Tuple
 
 import numpy as np
 import pytest
@@ -13,10 +11,12 @@ from astropy.time import TimeDelta
 from dkist_header_validator import spec122_validator
 from dkist_processing_common._util.scratch import WorkflowFileSystem
 from dkist_processing_common.models.tags import Tag
+from dkist_processing_common.tests.conftest import FakeGQLClient
 
+from dkist_processing_visp.models.tags import VispStemName
 from dkist_processing_visp.models.tags import VispTag
+from dkist_processing_visp.tasks.science import CalibrationCollection
 from dkist_processing_visp.tasks.science import ScienceCalibration
-from dkist_processing_visp.tests.conftest import FakeGQLClient
 from dkist_processing_visp.tests.conftest import generate_fits_frame
 from dkist_processing_visp.tests.conftest import VispConstantsDb
 from dkist_processing_visp.tests.conftest import VispHeadersValidObserveFrames
@@ -25,7 +25,7 @@ from dkist_processing_visp.tests.conftest import VispTestingParameters
 
 @dataclass
 class VispScienceTestingParameters(VispTestingParameters):
-    visp_beam_border: int = 10
+    visp_beam_border: int = 25  # Needs to be half of `array_shape` below
 
 
 @pytest.fixture(scope="function", params=["Full Stokes", "Stokes-I"])
@@ -42,8 +42,8 @@ def science_calibration_task(
     else:
         num_modstates = 1
         polarimeter_mode = "observe_intensity"
-    array_shape = (1, 20, 10)
-    intermediate_shape = (10, 10)
+    array_shape = (1, 50, 20)
+    intermediate_shape = (25, 20)
     dataset_shape = (num_beams * num_map_scans * num_raster_steps * num_modstates,) + array_shape[
         1:
     ]
@@ -84,7 +84,7 @@ def science_calibration_task(
 
             # Create fake geometric objects
             angle = np.array([0.0])
-            offset = np.array([0.0, 0.0])
+            offset = np.array([-10.2, 5.1])
             spec_shift = np.zeros(intermediate_shape[0])
             for beam in range(1, num_beams + 1):
                 task.intermediate_frame_helpers_write_arrays(
@@ -104,10 +104,27 @@ def science_calibration_task(
                     all_zeros, beam=beam, task="DARK", exposure_time=exposure_time
                 )
 
-            # Create fake solar_gain intermediate arrays
+                if request.param == "Full Stokes":
+                    # BackgroundLight object
+                    task.intermediate_frame_helpers_write_arrays(
+                        arrays=all_zeros, beam=beam, task="BACKGROUND"
+                    )
+
+            # Create fake lamp and solar gain intermediate arrays
             for beam in range(1, num_beams + 1):
                 for modstate in range(1, num_modstates + 1):
                     gain_hdul = fits.HDUList([fits.PrimaryHDU(data=all_ones)])
+                    task.fits_data_write(
+                        hdu_list=gain_hdul,
+                        tags=[
+                            VispTag.intermediate(),
+                            VispTag.frame(),
+                            VispTag.task("LAMP_GAIN"),
+                            VispTag.beam(beam),
+                            VispTag.modstate(modstate),
+                        ],
+                    )
+
                     task.fits_data_write(
                         hdu_list=gain_hdul,
                         tags=[
@@ -144,6 +161,7 @@ def science_calibration_task(
                         hdul = generate_fits_frame(
                             header_generator=header_generator, shape=array_shape
                         )
+                        header = hdul[0].header
                         task.fits_data_write(
                             hdu_list=hdul,
                             tags=[
@@ -157,7 +175,7 @@ def science_calibration_task(
                             ],
                         )
 
-            yield task, request.param
+            yield task, request.param, offset, header, intermediate_shape
         except:
             raise
         finally:
@@ -166,7 +184,7 @@ def science_calibration_task(
 
 
 @pytest.fixture(scope="session")
-def headers_with_dates() -> Tuple[List[fits.Header], str, int, int]:
+def headers_with_dates() -> tuple[list[fits.Header], str, int, int]:
     num_headers = 5
     start_time = "1969-12-06T18:00:00"
     exp_time = 12
@@ -195,17 +213,36 @@ def headers_with_dates() -> Tuple[List[fits.Header], str, int, int]:
 
 
 @pytest.fixture(scope="session")
-def compressed_headers_with_dates(headers_with_dates) -> Tuple[List[fits.Header], str, int, int]:
+def compressed_headers_with_dates(headers_with_dates) -> tuple[list[fits.Header], str, int, int]:
     headers, start_time, exp_time, time_delta = headers_with_dates
     comp_headers = [fits.hdu.compressed.CompImageHeader(h, h) for h in headers]
     return comp_headers, start_time, exp_time, time_delta
 
 
+@pytest.fixture(scope="function")
+def calibration_collection_with_geo_shifts() -> tuple[CalibrationCollection, np.ndarray]:
+    shifts = np.random.random((2, 10, 2)) * 100.0 - 50
+    geo_shifts = {str(b + 1): {f"m{m + 1}": shifts[b, m, :] for m in range(10)} for b in range(2)}
+    return (
+        CalibrationCollection(
+            dark=dict(),
+            background=dict(),
+            lamp_gain=dict(),
+            solar_gain=dict(),
+            angle=dict(),
+            state_offset=geo_shifts,
+            spec_shift=dict(),
+            demod_matrices=None,
+        ),
+        shifts,
+    )
+
+
 def test_science_calibration_task(science_calibration_task, mocker):
     """
     Given: A ScienceCalibration task
     When: Calling the task instance
-    Then: There are the expected number of science frames with the correct tags applied
+    Then: There are the expected number of science frames with the correct tags applied and the headers have been correctly updated
     """
 
     mocker.patch(
@@ -213,9 +250,16 @@ def test_science_calibration_task(science_calibration_task, mocker):
     )
 
     # When
-    task, polarization_mode = science_calibration_task
+    task, polarization_mode, offset, og_header, og_single_beam_shape = science_calibration_task
     task()
 
+    # 1 from re-dummification
+    expected_final_shape = (
+        1,
+        og_single_beam_shape[0] - np.ceil(np.abs(offset[0])),
+        og_single_beam_shape[1] - np.ceil(np.abs(offset[1])),
+    )
+
     # Then
     tags = [
         VispTag.calibrated(),
@@ -230,10 +274,23 @@ def test_science_calibration_task(science_calibration_task, mocker):
         assert len(files) == 4
     for file in files:
         hdul = fits.open(file)
-        assert hdul[0].data.shape == (1, 10, 10)  # 1 from re-dummification
+        assert hdul[0].data.shape == expected_final_shape
         assert "DATE-BEG" in hdul[0].header.keys()
         assert "DATE-END" in hdul[0].header.keys()
 
+        # Check that map scan keys were updated
+        map_scan = [
+            int(t.split("_")[-1]) for t in task.tags(file) if VispStemName.map_scan.value in t
+        ][0]
+        assert hdul[0].header["VSPNMAPS"] == 2
+        assert hdul[0].header["VSPMAP"] == map_scan
+
+        # Check that WCS keys were updated
+        if offset[0] > 0:
+            assert hdul[0].header["CRPIX2"] == og_header["CRPIX2"] - np.ceil(offset[0])
+        if offset[1] > 0:
+            assert hdul[0].header["CRPIX1"] == og_header["CRPIX1"] - np.ceil(offset[1])
+
     quality_files = task.read(tags=[Tag.quality("TASK_TYPES")])
     for file in quality_files:
         with file.open() as f:
@@ -310,3 +367,32 @@ def test_compute_date_keys_compressed_headers(
 
     assert final_header_from_single["DATE-BEG"] == headers[0]["DATE-BEG"]
     assert final_header_from_single["DATE-END"] == date_end_from_single
+
+
+def test_beam_overlap_slice(calibration_collection_with_geo_shifts):
+    """
+    Given: A CalibrationCollection object with populated state_offsets
+    When: Computing the overlapping beam slices
+    Then: The correct values are returned
+    """
+    calibrations, shifts = calibration_collection_with_geo_shifts
+    flat_shifts = shifts.reshape((shifts.shape[0] * shifts.shape[1], shifts.shape[2]))
+    x_max = flat_shifts[np.argmax(np.abs(flat_shifts[:, 0])), 0]
+    y_max = flat_shifts[np.argmax(np.abs(flat_shifts[:, 1])), 1]
+    x_slice, y_slice = calibrations.beams_overlap_slice
+
+    assert x_slice.step is None
+    assert y_slice.step is None
+    if x_max < 0:
+        assert x_slice.start is None
+        assert x_slice.stop == np.floor(x_max)
+    else:
+        assert x_slice.start == np.ceil(x_max)
+        assert x_slice.stop is None
+
+    if y_max < 0:
+        assert y_slice.start is None
+        assert y_slice.stop == np.floor(y_max)
+    else:
+        assert y_slice.start == np.ceil(y_max)
+        assert y_slice.stop is None
