@@ -47,7 +47,10 @@ def write_l1_task(
         try:  # This try... block is here to make sure the dbs get cleaned up if there's a failure in the fixture
             stokes_params = ["I", "Q", "U", "V"]
             used_stokes_params = []
-            hdu = fits.PrimaryHDU(data=np.ones(shape=(128, 128, 1)), header=calibrated_visp_dataset)
+            # Random data needed so skew and kurtosis don't barf
+            hdu = fits.PrimaryHDU(
+                data=np.random.random((128, 128, 1)) * 100.0, header=calibrated_visp_dataset
+            )
             hdul = fits.HDUList([hdu])
             hdul[0].header["CTYPE1"] = wcs_axis_names[0]
             hdul[0].header["CTYPE2"] = wcs_axis_names[1]
@@ -87,6 +90,9 @@ def test_write_l1_frame(write_l1_task, mocker, wcs_axis_names):
             assert len(hdu_list) == 2  # Primary, CompImage
             assert type(hdu_list[0]) is fits.PrimaryHDU
             assert type(hdu_list[1]) is fits.CompImageHDU
+            assert header["DTYPE1"] == "SPATIAL"
+            assert header["DTYPE2"] == "SPECTRAL"
+            assert header["DTYPE3"] == "SPATIAL"
             assert header["DAAXES"] == 2
             if len(stokes_params) == 1:
                 assert "DNAXIS4" not in header
@@ -121,8 +127,3 @@ def test_write_l1_frame(write_l1_task, mocker, wcs_axis_names):
                 + Time(header["DATE-BEG"], precision=6)
             ).isot
             assert header["DATE-AVG"] == date_avg
-
-            # Check WCS axes are the right way around by checking the CTYPEs
-            assert header["CTYPE1"] == "HPLT-TAN"
-            assert header["CTYPE2"] == "AWAV"
-            assert header["CTYPE3"] == "HPLN-TAN"
